from datetime import datetime
import webbrowser
import difflib
import os.path


dirname = os.path.dirname(__file__)

def valid_input(f1, f2):
    print("Opening files")
    if os.path.isfile(f1) and os.path.isfile(f2):
        return True
    else:
        if not os.path.isfile(f1):
            print(f"File 1 '{f1}' does not exist")
        if not os.path.isfile(f2):
            print(f"File 2 '{f2}' does not exist")
        return False


def compare_files(f1path, f2path):
    # Open Files in Read Mode
    f1 = open(f1path, 'r')
    f2 = open(f2path, 'r')
    f1list = f1.readlines()
    f2list = f2.readlines()

    print("Comparing reports")

    # delta = difflib.unified_diff(f1list, f2list, f1.name, f2.name)
    delta = difflib.Differ().compare(f1list, f2list)

    print("Building comparison report")

    resultlines = []

    for line in delta:
        if line.startswith("  "):  # línea común, remover espacio extra
            line = line[2:]
        if line.startswith("- "):  # línea solo file1
            line = line[2:]
            if "<tr>" in line:
                pos = line.index("<tr>")
                line = line[:pos+3] + " class='before'" + line[pos+3:]
        if line.startswith("+ "):  # línea solo file2
            line = line[2:]
            if "<tr>" in line:
                pos = line.index("<tr>")
                line = line[:pos+3] + " class='after'" + line[pos+3:]
        if not line.startswith("? "):
            resultlines.append(line)

    f1.close()
    f2.close()

    return resultlines


def build_report(reportlines):
    try:
        comparisondate = datetime.now().strftime("%Y_%-m_%d--%H:%M")
    except ValueError:
        comparisondate = datetime.now().strftime("%Y_%m_%d--%H-%M")
	
    filename = f"Report_Comparison_{comparisondate}"
    fileroute = os.path.join(dirname, "..", "..", "reports", f"{filename}.html")
    # htmlfile = open(f'../reports/{filename}.html', "w")
    htmlfile = open(fileroute, "w")
    htmlfile.write("".join(reportlines))
    htmlfile.close()

    print("Opening result file, please wait")

    webbrowser.open_new_tab(fileroute)


def main(f1path, f2path):
    if valid_input(f1path, f2path):
        build_report(compare_files(f1path, f2path))
