import ipaddress
import nmap
from . import PortScanner


def main(network_range_input):
    try:
        targets = input_validation(network_range_input)
        if targets is not None:
            if len(targets) > 0:
                scan_targets(targets)
            else:
                print(f"No hosts to be scanned in {network_range_input}")
    except ipaddress.NetmaskValueError:
        print(f'Wrong netmask input format: \'{network_range_input}\' is not a valid IP address/CIDR block')
    except ipaddress.AddressValueError:
        print(f'Wrong address input format: \'{network_range_input}\' is not a valid IP address/CIDR block')
    except ValueError:
        print(f'Wrong input format: the \'{network_range_input}\' network has host bits set')


def input_validation(network_range_input):
    iplist = [str(ip) for ip in ipaddress.IPv4Network(network_range_input).hosts()]  # .hosts()?
    print(f"Network Scan started: {len(iplist)} IPs pending to be scanned")
    print("Please, be patient - This process may take some time")
    return iplist


def scan_targets(ips, testmode=False):
    active_hosts = []
    scanner = nmap.PortScanner()
    ip_count = 0
    quarter = round(len(ips) / 4)

    for ip in ips:
        scanner.scan(ip, "1", "-v")
        if scanner[ip].state() == 'up':
            print(f"    Host {ip} is {scanner[ip].state()}")
            active_hosts.append(ip)

        # progress update for long IP ranges
        if len(ips) > 20:
            ip_count = ip_count + 1
            if ip_count / quarter == 1:
                print("Scan Progress: 25%")
            if ip_count / (quarter * 2) == 1:
                print("Scan Progress: 50%")
            if ip_count / (quarter * 3) == 1:
                print("Scan Progress: 75%")

    if testmode:
        return active_hosts

    if len(active_hosts) > 0:
        print(f"Active hosts found: {len(active_hosts)}")

        PortScanner.main(active_hosts)
    else:
        print(f"No hosts found")


