from src.oop.Port import Port
from src.oop.Header import Header
from src.oop.Host import Host
import nmap
import requests
import warnings
from datetime import datetime
from . import ReportBuilder, HeaderParser


def scan_ports(targetip):
    print(f"Scan of target {targetip} started")

    scanner = nmap.PortScanner()
    scanner.scan(targetip, arguments="-n -sT -sV -p 80,443")
    portnumlist = []

    if "tcp" in scanner[targetip].keys():
        for port in scanner[targetip]["tcp"]:
            if scanner[targetip]["tcp"][port]["name"] == "http":
                print(f"    {port} has a HTTP service")
                portnumlist.append(port)

    portlist = []

    with warnings.catch_warnings():  # evitar warning de no verificar certificado ssl
        warnings.simplefilter("ignore")
        for portnum in portnumlist:
            recvheaders = headerrequest(targetip, portnum)
            if recvheaders:
                comparedheaders = headercomparison(recvheaders)
                portlist.append(Port(portnum, comparedheaders))

    if portlist:  # port list is not empty
        return Host(targetip, portlist)
    else:  # port list is empty, don't generate report
        print("No HTTP/HTTPS ports found")
        return None


def headerrequest(ip, port):
    r = None
    try:
        r = requests.head(f"http://{ip}:{port}")
    except requests.exceptions.RequestException:
        try:
            r = requests.get(f"http://{ip}:{port}")
        except requests.exceptions.RequestException:
            try:
                r = requests.head(f"https://{ip}:{port}", verify=False)
            except requests.exceptions.RequestException:
                try:
                    r = requests.head(f"https://{ip}:{port}", verify=False)
                except requests.exceptions.RequestException:
                    print(f"    Can't get headers from {ip}:{port}")
                    pass

    if r is not None:
        return r.headers


def headercomparison(headerstocheck):
    expectedheaders = HeaderParser.main()
    expectednames = []
    # if the expected header is not in the list of obtained headers, the header is not implemented
    for expectedheader in expectedheaders:
        expectednames.append(expectedheader.name.upper())
        if expectedheader.name.upper() in map(str.upper, headerstocheck):
            expectedheader.status = "DEFINED"
            expectedheader.responsevalue = headerstocheck[expectedheader.name.upper()]
        else:
            expectedheader.status = "NOT DEFINED"

    # if the obtained header is not in the list of expected headers, the header is custom (and possibly dangerous)
    for obtainedheader in headerstocheck:
        if obtainedheader.upper() not in expectednames:
            expectedheaders.append(Header(obtainedheader.upper(), "DEFINED", "Custom",
                                          None, None, None, headerstocheck[obtainedheader]))

    return expectedheaders


def main(targetiplist):
    timestartscan = datetime.now()
    hostlist = []
    for targetip in targetiplist:
        host = scan_ports(targetip)
        if host is not None:
            hostlist.append(host)

    timefinishscan = datetime.now()
    timetoscan = timefinishscan - timestartscan

    if hostlist:
        ReportBuilder.main(hostlist, timetoscan)
    else:
        print("No hosts with HTTP/HTTPS services found")




