from datetime import datetime
import webbrowser
import re
import os

dirname = os.path.dirname(__file__)
filenamesep = os.path.join(dirname, "..", "HTML", "htmlseparatedreporttemplate.txt")
filenamejoined = os.path.join(dirname, "..", "HTML", "htmljoinedreporttemplate.txt")
filenamehostrow = os.path.join(dirname, "..", "HTML", "htmlhostrow.txt")
filenamehostpar = os.path.join(dirname, "..", "HTML", "htmlhostparagraph.txt")
filenameheadtab = os.path.join(dirname, "..", "HTML", "htmlheadertable.txt")
filenameheadrow = os.path.join(dirname, "..", "HTML", "htmlheaderrow.txt")

with open(filenamesep, "r") as html_separated_report_text:
    htmlseparatedreportstr = html_separated_report_text.read()
with open(filenamejoined, "r") as html_joined_report_text:
    htmljoinedreportstr = html_joined_report_text.read()
with open(filenamehostrow, "r") as html_host_row:
    htmlstr_host_row = html_host_row.read()
with open(filenamehostpar, "r") as html_host_paragraph:
    htmlstr_host_paragraph = html_host_paragraph.read()
with open(filenameheadtab, "r") as html_header_table:
    htmlstr_header_table = html_header_table.read()
with open(filenameheadrow, "r") as html_header_row:
    htmlstr_header_row = html_header_row.read()


def separated_reports(hostlist):  # 1 host = 1 report; many hosts = many reports
    for host in hostlist:
        porttables = []
        sec_headers_total = 0
        sec_header_implemented = 0
        for port in host.ports:
            tablerows = []
            for header in port.headers:
                if header.category == "Security":
                    sec_headers_total += 1
                    if header.status == "DEFINED":
                        if header.correctvalues and header.responsevalue is not None:  # list not empty
                            correctvalue = False
                            for cv in header.correctvalues:
                                if re.match(cv, header.responsevalue):
                                    correctvalue = True
                            if correctvalue:
                                sec_header_implemented += 1
                htmlstatus = ""
                htmltype = ""
                htmlreference = ""
                htmlresponsevalue = ""

                if header.status == "DEFINED" or header.category == "Security":
                    # status color
                    if header.status == "DEFINED":
                        htmlstatus = "class='correct-value'>DEFINED"
                    elif header.status == "NOT DEFINED":
                        htmlstatus = "class='incorrect-value'>NOT DEFINED"
                    # type color
                    if header.headertype == "Standard":
                        htmltype = "class='correct-value'>Standard"
                    else:
                        htmltype = f"class='manual-rev-value'>{header.headertype}"
                    # response value color
                    if header.correctvalues and header.responsevalue is not None:  # list not empty
                        for cv in header.correctvalues:
                            if re.match(cv, header.responsevalue):
                                htmlresponsevalue = f"class='correct-value'>{header.responsevalue}"
                    else:
                        if header.responsevalue is not None:
                            htmlresponsevalue = f"class='manual-rev-value'>{header.responsevalue}"
                        else:
                            htmlresponsevalue = f">{header.responsevalue}"

                    # reference or not
                    if header.reference:
                        htmlreference = f"<a href=\"{header.reference}\">Link</a>"
                    else:
                        htmlreference = "-"

                    # add row
                    tablerows.append(htmlstr_header_row.format(headername=header.name, headerstatus=htmlstatus,
                                                               headertype=htmltype, headercategory=header.category,
                                                               headerresponsevalue=htmlresponsevalue,
                                                               headerreference=htmlreference))

            porttables.append(htmlstr_header_table.format(portnum=port.number, tablerows="\n".join(tablerows)))

        now = datetime.now()
        #scandate = now.strftime("%d %b %Y %H:%M")
        #filename = f'Scan_Report_{host.ipaddress}_{now.strftime("%Y_%-m_%d--%H:%M")}_' \
        #           f'Headers:{sec_header_implemented}of{sec_headers_total}_'
        try:
            scandate = now.strftime("%d %b %Y %H:%M") # works on linux
            filename = f'Scan_Report_{host.ipaddress}_{now.strftime("%Y_%-m_%d--%H:%M")}_Headers:{sec_header_implemented}of{sec_headers_total}'
        except ValueError:
            scandate = now.strftime("%d %m %Y %H:%M") # works on windows?
            filename = f'Scan_Report_{host.ipaddress}_{now.strftime("%Y_%m_%d--%H-%M")}_Headers{sec_header_implemented}of{sec_headers_total}'

        fileroute = os.path.join(dirname, "..", "..", "reports", f"{filename}.html")
        # htmlfile = open(f'../reports/{filename}.html', "w")
        htmlfile = open(fileroute, "w")
        htmlfile.write(htmlseparatedreportstr.format(ipaddress=host.ipaddress, scandate=scandate,
                                                     porttables="".join(porttables)))
        htmlfile.close()
        # webbrowser.open_new_tab(f'../../reports/{filename}.html')
        webbrowser.open_new_tab(fileroute)


def joined_report(hostlist, scantime):  # 1 host = 1 report; many hosts = 1 report
    hostrows = []
    hostparagraphs = []
    sec_headers_total = 0
    sec_header_implemented = 0
    for host in hostlist:
        hostrows.append(htmlstr_host_row.format(ipaddress=host.ipaddress))
        porttables = []
        for port in host.ports:
            tablerows = []
            for header in port.headers:
                if header.category == "Security":
                    sec_headers_total += 1
                    if header.status == "DEFINED":
                        if header.correctvalues and header.responsevalue is not None:  # list not empty
                            correctvalue = False
                            for cv in header.correctvalues:
                                if re.match(cv, header.responsevalue):
                                    correctvalue = True
                            if correctvalue:
                                sec_header_implemented += 1
                htmlstatus = ""
                htmltype = ""
                htmlreference = ""
                htmlresponsevalue = ""

                if header.status == "DEFINED" or header.category == "Security":
                    # status color
                    if header.status == "DEFINED":
                        htmlstatus = "class='correct-value'>DEFINED"
                    elif header.status == "NOT DEFINED":
                        htmlstatus = "class='incorrect-value'>NOT DEFINED"
                    # type color
                    if header.headertype == "Standard":
                        htmltype = "class='correct-value'>Standard"
                    else:
                        htmltype = f"class='manual-rev-value'>{header.headertype}"
                    # response value color
                    if header.correctvalues and header.responsevalue is not None:  # list not empty
                        for cv in header.correctvalues:
                            if re.match(cv, header.responsevalue):
                                htmlresponsevalue = f"class='correct-value'>{header.responsevalue}"
                    else:
                        if header.responsevalue is not None:
                            htmlresponsevalue = f"class='manual-rev-value'>{header.responsevalue}"
                        else:
                            htmlresponsevalue = f">{header.responsevalue}"

                    # reference or not
                    if header.reference:
                        htmlreference = f"<a href=\"{header.reference}\">Link</a>"
                    else:
                        htmlreference = "-"

                    # add row
                    tablerows.append(htmlstr_header_row.format(headername=header.name, headerstatus=htmlstatus,
                                                               headertype=htmltype, headercategory=header.category,
                                                               headerresponsevalue=htmlresponsevalue,
                                                               headerreference=htmlreference))

            porttables.append(htmlstr_header_table.format(portnum=port.number, tablerows="\n".join(tablerows)))

        hostparagraphs.append(htmlstr_host_paragraph.format(ipaddress=host.ipaddress, porttables="\n".join(porttables)))

    now = datetime.now()
    #scandate = now.strftime("%d %b %Y %H:%M")
    #filename = f'Scan_Report_{now.strftime("%Y_%-m_%d--%H:%M")}_Headers:{sec_header_implemented}of{sec_headers_total}'
    try:
        scandate = now.strftime("%d %b %Y %H:%M")  # works on linux
        filename = f'Scan_Report_{now.strftime("%Y_%-m_%d--%H:%M")}_Headers:{sec_header_implemented}of{sec_headers_total}'
    except ValueError:
        scandate = now.strftime("%d %m %Y %H:%M")  # works on windows?
        filename = f'Scan_Report_{now.strftime("%Y_%m_%d--%H-%M")}_Headers{sec_header_implemented}of{sec_headers_total}'

    fileroute = os.path.join(dirname, "..", "..", "reports", f"{filename}.html")
    # htmlfile = open(f'./reports/{filename}.html', "w")
    htmlfile = open(fileroute, "w")
    htmlfile.write(htmljoinedreportstr.format(scandate=scandate, timetaken=scantime,
                                              hostparagraphs="".join(hostparagraphs),
                                              hostrows="\n".join(hostrows)))
    htmlfile.close()
    webbrowser.open_new_tab(fileroute)


# Receives a list of hosts found in a scan, the time taken to perform the scan,
# and builds an HTML document that shows all the data retrieved. If there are
# two or more hosts, also builds an individual document for each of them
def main(hostlist, scantime=0):
    if len(hostlist) == 1:
        print("Building scan report, please wait")
        separated_reports(hostlist)
    elif len(hostlist) > 1:  # 2 or more hosts -> additional single reports -> nueva carpeta?
        print("Building scan reports, please wait")
        joined_report(hostlist, scantime)
        separated_reports(hostlist)


