import unittest
import src.reportcomparator.ReportComparator as rc
import os

dirname = os.path.dirname(__file__)
file1route = os.path.join(dirname, ".", "testfiles", "scanreporttest1.html")
file2route = os.path.join(dirname, ".", "testfiles", "scanreporttest2.html")
file3route = os.path.join(dirname, ".", "testfiles", "scanreporttest3.html")
file4route = os.path.join(dirname, ".", "testfiles", "scanreporttest4.html")
file5route = os.path.join(dirname, ".", "testfiles", "scanreporttest5.html")


class TestReportComparatorMethods(unittest.TestCase):

    def test_valid_input(self):
        # neither file exist
        self.assertFalse(rc.valid_input("invalid/file1/route", "invalid/file2/route"))
        # only first file exists
        self.assertFalse(rc.valid_input(file1route, "invalid/file2/route"))
        # only second file exists
        self.assertFalse(rc.valid_input("invalid/file1/route", file2route))
        # empty routes
        self.assertFalse(rc.valid_input("", ""))
        self.assertFalse(rc.valid_input(file1route, ""))
        self.assertFalse(rc.valid_input("", file2route))
        # both files exist
        self.assertTrue(rc.valid_input(file1route, file2route))


    def test_compare_files(self):
        before = "<tr class='before'>"
        after = "<tr class='after'>"
        # both files are the same (no differences)->no lines that include '<tr class="before">' or '<tr class="after">'
        resultlines1 = rc.compare_files(file1route, file1route)
        for line in resultlines1:
            self.assertTrue(before not in line or after not in line)

        # there are differences between the two files->4 modifications -> 4 before + 4 after = 8
        resultlines2 = rc.compare_files(file1route, file2route)
        counter1 = 0
        for line in resultlines2:
            if before in line or after in line:
                counter1 += 1
        self.assertEqual(counter1, 8)

        # there are differences between the two files->2 element removed -> 2 before = 2
        resultlines3 = rc.compare_files(file1route, file3route)
        counter2 = 0
        for line in resultlines3:
            if before in line or after in line:
                counter2 += 1
        self.assertEqual(counter2, 2)

        # there are differences between the two files->2 element added -> 2 after = 2
        resultlines4 = rc.compare_files(file1route, file4route)
        counter3 = 0
        for line in resultlines4:
            if before in line or after in line:
                counter3 += 1
        self.assertEqual(counter3, 2)

        # there are differences between the two files->2 modified 2 added 2 removed -> 4 after + 4 before = 8
        resultlines5 = rc.compare_files(file1route, file5route)
        counter4 = 0
        for line in resultlines5:
            if before in line or after in line:
                counter4 += 1
        self.assertEqual(counter4, 8)


if __name__ == '__main__':
    unittest.main()
