import ipaddress
import unittest
import src as nsc


class TestNetworkScannerMethods(unittest.TestCase):

    def test_input_validation(self):
        with self.assertRaises(ipaddress.NetmaskValueError):
            nsc.input_validation("192.168.32.0/-1")
        with self.assertRaises(ipaddress.NetmaskValueError):
            nsc.input_validation("192.168.32.0/-50")
        with self.assertRaises(ipaddress.NetmaskValueError):
            nsc.input_validation("192.168.32.0/-999999")
        with self.assertRaises(ipaddress.NetmaskValueError):
            nsc.input_validation("192.168.32.0/33")
        with self.assertRaises(ipaddress.NetmaskValueError):
            nsc.input_validation("192.168.32.0/50")
        with self.assertRaises(ipaddress.NetmaskValueError):
            nsc.input_validation("192.168.32.0/999999")

        #self.assertEqual(len(nsc.input_validation("0.0.0.0/1")), 2147483648)  # extremadamente lento, saltar
        self.assertEqual(len(nsc.input_validation("192.168.32.0/32")), 1)
        self.assertEqual(len(nsc.input_validation("192.168.32.0/20")), 4094)

        with self.assertRaises(ipaddress.AddressValueError):
            nsc.input_validation("256.168.32.0/27")
        with self.assertRaises(ipaddress.AddressValueError):
            nsc.input_validation("400.168.32.0/27")
        with self.assertRaises(ipaddress.AddressValueError):
            nsc.input_validation("999999.168.32.0/27")

        with self.assertRaises(ipaddress.AddressValueError):
            nsc.input_validation("192.256.32.0/27")
        with self.assertRaises(ipaddress.AddressValueError):
            nsc.input_validation("192.400.32.0/27")
        with self.assertRaises(ipaddress.AddressValueError):
            nsc.input_validation("192.999999.32.0/27")

        with self.assertRaises(ipaddress.AddressValueError):
            nsc.input_validation("192.168.256.0/27")
        with self.assertRaises(ipaddress.AddressValueError):
            nsc.input_validation("192.168.400.0/27")
        with self.assertRaises(ipaddress.AddressValueError):
            nsc.input_validation("192.168.999999.0/27")

        with self.assertRaises(ipaddress.AddressValueError):
            nsc.input_validation("192.168.32.256/27")
        with self.assertRaises(ipaddress.AddressValueError):
            nsc.input_validation("192.168.32.400/27")
        with self.assertRaises(ipaddress.AddressValueError):
            nsc.input_validation("192.168.32.999999/27")

        with self.assertRaises(ValueError):
            nsc.input_validation("192.168.255.0/23")
        with self.assertRaises(ValueError):
            nsc.input_validation("192.168.255.0/22")
        with self.assertRaises(ValueError):
            nsc.input_validation("192.168.255.0/13")

        with self.assertRaises(ValueError):
            nsc.input_validation("192.168.32.0/0")
        with self.assertRaises(ValueError):
            nsc.input_validation("192.168.255.192/25")
        with self.assertRaises(ValueError):
            nsc.input_validation("192.168.255.192/24")
        with self.assertRaises(ValueError):
            nsc.input_validation("192.168.255.192/12")

    def test_scan_targets(self):
        # requiere una máquina virtual en la red con ip 192.168.33.20
        hosts1 = nsc.scan_targets([], testmode=True)
        self.assertEqual(len(hosts1), 0)
        hosts2 = nsc.scan_targets(["192.168.33.20"], testmode=True)
        self.assertEqual(len(hosts2), 1)
        hosts3 = nsc.scan_targets(["192.168.33.22"], testmode=True)
        self.assertEqual(len(hosts3), 1)
        hosts4 = nsc.scan_targets(["192.168.33.17", "192.168.33.18", "192.168.33.19", "192.168.33.20", "192.168.33.21"],
                                  testmode=True)
        self.assertEqual(len(hosts4), 1)
        hosts5 = nsc.scan_targets(["192.168.33.11", "192.168.33.12", "192.168.33.13", "192.168.33.14", "192.168.33.15",
                                   "192.168.33.16", "192.168.33.17"], testmode=True)
        self.assertEqual(len(hosts5), 0)
        hosts6 = nsc.scan_targets(["192.168.33.21", "192.168.33.22", "192.168.33.23", "192.168.33.24", "192.168.33.25",
                                   "192.168.33.26", "192.168.33.27"], testmode=True)
        self.assertEqual(len(hosts6), 1)
        hosts7 = nsc.scan_targets(["192.168.33.18", "192.168.33.19", "192.168.33.20", "192.168.33.21", "192.168.33.22",
                                   "192.168.33.23", "192.168.33.24", "192.168.33.25"], testmode=True)
        self.assertEqual(len(hosts7), 2)


if __name__ == '__main__':
    unittest.main()

