import unittest
import src as psc


class TestPortScannerMethods(unittest.TestCase):

    def test_header_request(self):
        # requiere una máquina virtual en la red con ip 192.168.33.20 y el test server 1
        headers1 = psc.headerrequest("192.168.33.20", 8000)
        self.assertIsNotNone(headers1)
        self.assertEqual(len(headers1), 6)
        self.assertIsNotNone(headers1["Server"])
        self.assertIsNotNone(headers1["Date"])
        self.assertIsNotNone(headers1["Content-Type"])
        self.assertIsNotNone(headers1["Custom-header-here"])
        self.assertIsNotNone(headers1["otro-custom"])
        self.assertIsNotNone(headers1["X-Content-Type-Options"])

    def test_header_comparison(self): # usa headerrequest y headerparser
        # requiere una máquina virtual en la red con ip 192.168.33.20 y el test server 1
        headersdic = psc.headerrequest("192.168.33.20", 8000)
        headers = psc.headercomparison(headersdic)
        self.assertEqual(len(headers), 71)  # 69 por defecto + 2 custom
        self.assertTrue(any(h.name == "SERVER" and h.status == "DEFINED" for h in headers))
        self.assertTrue(any(h.name == "DATE" and h.status == "DEFINED" for h in headers))
        self.assertTrue(any(h.name == "CONTENT-TYPE" and h.status == "DEFINED" for h in headers))
        self.assertTrue(any(h.name == "CUSTOM-HEADER-HERE" and h.status == "DEFINED" for h in headers))
        self.assertTrue(any(h.name == "OTRO-CUSTOM" and h.status == "DEFINED" for h in headers))
        self.assertTrue(any(h.name == "X-CONTENT-TYPE-OPTIONS" and h.status == "DEFINED" for h in headers))
        self.assertFalse(any(h.name == "X-FRAME-OPTIONS" and h.status == "DEFINED" for h in headers))
        self.assertFalse(any(h.name == "X-XSS-PROTECTION" and h.status == "DEFINED" for h in headers))

    def test_scan_ports(self):  # usa headerrequest y headercomparison
        # requiere una máquina virtual en la red con ip 192.168.33.20 y el test server 1
        host = psc.scan_ports("192.168.33.20")
        self.assertIsNotNone(host)
        self.assertEqual(host.ipaddress, "192.168.33.20")
        self.assertEqual(len(host.ports), 1)
        self.assertEqual(host.ports[0].number, 8000)
        self.assertEqual(len(host.ports[0].headers), 71)  # 69 por defecto + 2 custom
